﻿#!perl -w
use Test::More;
use strict;
use DE_EPAGES::Core::API::Html qw(
    BalanceHTMLTags
    RemoveHtmlTags
    RemoveSuspiciousChars
    CloseOpenHTMLTags
    ReplaceURIWithURL
);
use utf8;

plan tests => 5;

## no critic qw(ProhibitLongLines)

subtest 'BalanceHTMLTags' => sub {
    plan tests => 6;
    my $SourceText  = '<p><h1><span color="#333311">tsteawt</span><strong><!-- pagebreak --></strong></h1><p>dfasdasdasdaasasfasfs</p>asdasda';
    my $GoalText    = '<p></p><h1><span color="#333311">tsteawt</span><strong><!-- pagebreak --></strong></h1><p>dfasdasdasdaasasfasfs</p>asdasda';
    is(BalanceHTMLTags($SourceText), $GoalText, 'headings and paragraphs');

    $SourceText  = '<p><table><tr><td>test</tr><tr><td><div><strong><!-- pagebreak -->';
    $GoalText    = '<p></p><table><tr><td>test</td></tr><tr><td><div><strong><!-- pagebreak --></strong></div></td></tr></table>';
    is(BalanceHTMLTags($SourceText), $GoalText, 'malformed table');

    $SourceText  = '<p>&nbsp;</p><h2></h2><h1>tsteawt<b></b></h1><p>dfasdasdasdaasasfasf<table class="TinyMCETable" style="border: 1px solid #e27c7c; width: 100px; height: 50px; background-color: #703030;"><tbody><tr><td><font color="#e5d5d5"></font></td><td style="border: 1px solid #ff8400; text-align: center;"><font color="#e5d5d5">sdfsf</font></td></tr><tr><td></td><td><i>assaasas</i></td></tr></tbody></table>s</p><p>asdasda</p><p><i style="background-color: #e00808;"><span style="text-decoration: underline;"><a href="?ObjectPath=/Shops/DemoShop/Categories/CustomerInformation">Wichtig<table class=" TinyMCETable" style="border: 1px solid #302020; width: 20px; height: 30px; background-color: #bfffbf;"><tbody><tr><td></td><td>asd</td></tr><tr><td></td><td>asdddasas<!-- pagebreak -->';
    $GoalText    = '<p>&nbsp;</p><h2></h2><h1>tsteawt<b></b></h1><p>dfasdasdasdaasasfasf</p><table class="TinyMCETable" style="border: 1px solid #e27c7c; width: 100px; height: 50px; background-color: #703030;"><tbody><tr><td><font color="#e5d5d5"></font></td><td style="border: 1px solid #ff8400; text-align: center;"><font color="#e5d5d5">sdfsf</font></td></tr><tr><td></td><td><i>assaasas</i></td></tr></tbody></table>s<p>asdasda</p><p><i style="background-color: #e00808;"><span style="text-decoration: underline;"><a href="?ObjectPath=/Shops/DemoShop/Categories/CustomerInformation">Wichtig</a></span></i></p><table class=" TinyMCETable" style="border: 1px solid #302020; width: 20px; height: 30px; background-color: #bfffbf;"><tbody><tr><td></td><td>asd</td></tr><tr><td></td><td>asdddasas<!-- pagebreak --></td></tr></tbody></table>';
    is(BalanceHTMLTags($SourceText), $GoalText, 'complex from tinymce');

    $SourceText = '<p class=test"><span style="test2"></p>';
    $GoalText   = '<p class="test&quot;"><span style="test2"></span></p>';
    is(BalanceHTMLTags($SourceText), $GoalText, 'missing quotes at the beginning');

    $SourceText = '<script type="text/javascript">jQuery("<script>")</script>';
    $GoalText   = '<script type="text/javascript">jQuery("<script>")</script>';
    is(BalanceHTMLTags($SourceText), $GoalText, 'with script tags');

    $SourceText  = '<p><h1><span color="#333311">tsteawt</span><strong><!-- pagebreak --></strong></h1><p>dfasdasdasdaasasfasfs</p>asdasda';
    $GoalText    = '<p></p><h1><span color="#333311">tsteawt</span><strong><!-- pagebreak --></strong></h1><p>dfasdasdasdaasasfasfs</p>asdasda';
    is(BalanceHTMLTags($SourceText), $GoalText, 'headings and paragraphs');
};

subtest 'CloseOpenHTMLTags' => sub {
    plan tests => 12;

    my $SourceText  = '<p><h1><span color="#333311">tsteawt</span><strong><!-- pagebreak --></strong></h1><p>dfasdasdasdaasasfasfs</p>asdasda';
    my $GoalText    = '<p><h1><span color="#333311">tsteawt</span><strong><!-- pagebreak --></strong></h1></p><p>dfasdasdasdaasasfasfs</p>asdasda';
    cmp_ok(CloseOpenHTMLTags($SourceText), 'eq', $GoalText, 'headings and paragraphs');

    $SourceText  = '<p><table><tr><td>test</tr><tr><td><div><strong><!-- pagebreak -->';
    $GoalText    = '<p><table><tr><td>test</td></tr><tr><td><div><strong><!-- pagebreak --></strong></div></td></tr></table></p>';
    cmp_ok(CloseOpenHTMLTags($SourceText), 'eq', $GoalText, 'malformed table');

    $SourceText  = '<p>&nbsp;</p><h2></h2><h1>tsteawt<b></b></h1><p>dfasdasdasdaasasfasf<table class="TinyMCETable" style="border: 1px solid #e27c7c; width: 100px; height: 50px; background-color: #703030;"><tbody><tr><td><font color="#e5d5d5"></font></td><td style="border: 1px solid #ff8400; text-align: center;"><font color="#e5d5d5">sdfsf</font></td></tr><tr><td></td><td><i>assaasas</i></td></tr></tbody></table>s</p><p>asdasda</p><p><i style="background-color: #e00808;"><span style="text-decoration: underline;"><a href="?ObjectPath=/Shops/DemoShop/Categories/CustomerInformation">Wichtig<table class=" TinyMCETable" style="border: 1px solid #302020; width: 20px; height: 30px; background-color: #bfffbf;"><tbody><tr><td></td><td>asd</td></tr><tr><td></td><td>asdddasas<!-- pagebreak -->';
    $GoalText    = '<p>&nbsp;</p><h2></h2><h1>tsteawt<b></b></h1><p>dfasdasdasdaasasfasf<table class="TinyMCETable" style="border: 1px solid #e27c7c; width: 100px; height: 50px; background-color: #703030;"><tbody><tr><td><font color="#e5d5d5"></font></td><td style="border: 1px solid #ff8400; text-align: center;"><font color="#e5d5d5">sdfsf</font></td></tr><tr><td></td><td><i>assaasas</i></td></tr></tbody></table>s</p><p>asdasda</p><p><i style="background-color: #e00808;"><span style="text-decoration: underline;"><a href="?ObjectPath=/Shops/DemoShop/Categories/CustomerInformation">Wichtig<table class=" TinyMCETable" style="border: 1px solid #302020; width: 20px; height: 30px; background-color: #bfffbf;"><tbody><tr><td></td><td>asd</td></tr><tr><td></td><td>asdddasas<!-- pagebreak --></td></tr></tbody></table></a></span></i></p>';
    cmp_ok(CloseOpenHTMLTags($SourceText), 'eq', $GoalText, 'complex from tinymce');

    $SourceText = '<p class="test><span style="test2"></p>';
    $GoalText   = '<p class="test><span style="test2"></span></p>';
    cmp_ok(CloseOpenHTMLTags($SourceText), 'eq', $GoalText, 'missing quotes at the end');

    $SourceText = '<p class="test test2 id="test><span style="test2"></p>';
    $GoalText   = '<p class="test test2 id="test><span style="test2"></span></p>';
    cmp_ok(CloseOpenHTMLTags($SourceText), 'eq', $GoalText, 'missing quotes with more values');

    $SourceText = '<p class=test"><span style="test2"></p>';
    $GoalText   = '<p class=test"><span style="test2"></span></p>';
    cmp_ok(CloseOpenHTMLTags($SourceText), 'eq', $GoalText, 'missing quotes at the beginning');

    $SourceText = '<script type="text/javascript">jQuery("<script>")</script>';
    $GoalText   = '<script type="text/javascript">jQuery("<script>")</script>';
    cmp_ok(CloseOpenHTMLTags($SourceText), 'eq', $GoalText, 'with script tags');

    # javascript event in html
    $SourceText = '<a href="?" onclick="openWindow(this.href+\'&amp;ViewAction=ViewPopup\',\'\',\'PopUp\');">Test</a>';
    $GoalText = '<a href="?" onclick="openWindow(this.href+\'&amp;ViewAction=ViewPopup\',\'\',\'PopUp\');">Test</a>';
    cmp_ok(CloseOpenHTMLTags($SourceText), 'eq', $GoalText, 'inline javascript on dom node');

    # javascript event in html
    $SourceText = 'Ich habe die <a href="?ObjectPath=#Shop.Path[url]/Categories/TermsAndConditions" onclick="openWindow(this.href+\'&amp;ViewAction=ViewPopup\',\'\',\'PopUp\');return false;">Allgemeinen Geschäftsbedingungen</a> gelesen und erkenne sie hiermit an.<!-- comment! -->';
    $GoalText = 'Ich habe die <a href="?ObjectPath=#Shop.Path[url]/Categories/TermsAndConditions" onclick="openWindow(this.href+\'&amp;ViewAction=ViewPopup\',\'\',\'PopUp\');return false;">Allgemeinen Geschäftsbedingungen</a> gelesen und erkenne sie hiermit an.<!-- comment! -->';
    cmp_ok(CloseOpenHTMLTags($SourceText), 'eq', $GoalText, 'inline javascript on dom node');

    # javascript event in html
    $SourceText = '/a_s_d/!#Hündchen Ölgemälde-ÉÊËÌÍ#{}(b+).создавать<a href="/a_s_d/!#Hündchen Ölgemälde-ÉÊËÌÍ#{}(b+).создавать?ObjectPath=#Shop.Path[url]/Categories/TermsAndConditions" onclick="openWindow(this.href+\'&amp;ViewAction=ViewPopup\',\'\',\'PopUp\');return false;">Allgemeinen Geschäftsbedingungen</a> gelesen und erkenne sie hiermit an.<!-- comment! -->';
    $GoalText = '/a_s_d/!#Hündchen Ölgemälde-ÉÊËÌÍ#{}(b+).создавать<a href="/a_s_d/!#Hündchen Ölgemälde-ÉÊËÌÍ#{}(b+).создавать?ObjectPath=#Shop.Path[url]/Categories/TermsAndConditions" onclick="openWindow(this.href+\'&amp;ViewAction=ViewPopup\',\'\',\'PopUp\');return false;">Allgemeinen Geschäftsbedingungen</a> gelesen und erkenne sie hiermit an.<!-- comment! -->';
    cmp_ok(CloseOpenHTMLTags($SourceText), 'eq', $GoalText, 'inline javascript on dom node');

    $SourceText = 'Обратите внимание на наши <div><a rel="nofollow" href="?ObjectPath=Categories/CustomerInformation&amp;amp;ViewAction=ViewPopup" target="MyPopup">правила отказа</a> и <a rel="nofollow" href="?ObjectPath=Categories/TermsAndConditions&amp;amp;ViewAction=ViewPopup" target="MyPopup">общие условия</a>.';
    $GoalText = 'Обратите внимание на наши <div><a rel="nofollow" href="?ObjectPath=Categories/CustomerInformation&amp;amp;ViewAction=ViewPopup" target="MyPopup">правила отказа</a> и <a rel="nofollow" href="?ObjectPath=Categories/TermsAndConditions&amp;amp;ViewAction=ViewPopup" target="MyPopup">общие условия</a>.</div>';
    cmp_ok(CloseOpenHTMLTags($SourceText), 'eq', $GoalText, 'russian text');

    $SourceText = '<script type="text/javascript">jQuery("<div>")</script>';
    $GoalText   = '<script type="text/javascript">jQuery("<div>")</script>';
    cmp_ok(CloseOpenHTMLTags($SourceText), 'eq', $GoalText, 'with javascript selector');
};

subtest 'RemoveHtmlTags', sub {
    plan tests => 4;

    my $SourceText  = <<'END_TEXT';
    <p>
      <h1> <span color="#333311">headline</span> <strong><!-- pagebreak --></strong></h1>
    <p>text1<br>text 2<hr />&nbsp;&nbsp;text3 &amp;</p>
    <script language="JScript">
      document.all.obsolete = 1;
    </script>
    <STYLE>
      document.all.obsolete = 1;
    </STYLE>
    fin.
END_TEXT
    my $GoalText = <<'END_TEXT';
headline
text1
text 2
  text3 &
fin.
END_TEXT
    is( RemoveHtmlTags($SourceText)."\n", $GoalText, 'RemoveHtmlTags' );

    $SourceText  = join('', map { chr($_) } (0..8))  .'This&nbsp;-<br/>-'
                  .join('', map { chr($_) } (9..32)) ."is&nbsp;<br/>\x{007F}Correct" ## no critic qw(ProhibitMagicNumbers)
                  .chr(9).'tab'
                  .chr(10).'CR'   ## no critic qw(ProhibitMagicNumbers)
                  .chr(13).'LF.'; ## no critic qw(ProhibitMagicNumbers)

    $GoalText = "This -\n- is \nCorrect tab CR LF.\n";
    is( RemoveHtmlTags($SourceText)."\n", $GoalText, 'Remove non printable characters' );

    is( RemoveHtmlTags('&lt;&gt;&amp;&apos;&quot;&euro;&nbsp;&szlig;'), '<>&\'"€ ß', 'Remove entities' );
    is( RemoveHtmlTags('&nbsp;&nbsp;&nbsp;<br />&nbsp;&nbsp;'), "   \n  ", 'Multiple non-breaking spaces' );
};

subtest 'RemoveSuspiciousChars', sub {
    plan tests => 3;
    is( RemoveSuspiciousChars('a&lt;b'), 'ab', 'RemoveSuspiciousChars(\'&lt;\')' );
    is( RemoveSuspiciousChars('a>b'), 'ab', 'RemoveSuspiciousChars(\'>\')' );
    is( RemoveSuspiciousChars('a&#00062;b'), 'ab', 'RemoveSuspiciousChars(\'#00062\')' );
};

subtest 'ReplaceURIWithURL', sub {
    plan tests => 10;

    my $aTags = undef;
    my $BaseURL = 'http://example.de';
    my $Test = sub {
        my ($SourceText, $GoalText, $Message) = @_;
        is( ReplaceURIWithURL($SourceText, $BaseURL, $aTags), $GoalText, $Message );
    };

    $Test->( '<div><a href="?Parameter=Value"><img src="WebRoot/test.png" /></a></div>',
        '<div><a href="http://example.de?Parameter=Value"><img src="http://example.de/WebRoot/test.png" /></a></div>',
        'replace uris with full url in a- and img-tags.' );

    $Test->( '<div><a href="http://example.de?Parameter=Value"><img src="WebRoot/test.png" /></a></div>',
        '<div><a href="http://example.de?Parameter=Value"><img src="http://example.de/WebRoot/test.png" /></a></div>',
        'do not replace correct urls.' );

    $BaseURL .= '/epages/ShopAlias.sf';
    $Test->( '<div><a href="?Parameter=Value"><img src="http://example.de/WebRoot/test.png" /></a></div>',
        '<div><a href="http://example.de/epages/ShopAlias.sf?Parameter=Value"><img src="http://example.de/WebRoot/test.png" /></a></div>',
        'replace uris with full url even for extended baseurls.' );

    $Test->( '<script src="/blablabla.js"></script>',
        '<script src="http://example.de/blablabla.js"></script>',
        'replace uris with full url in script sources.' );

    $Test->( '<a href="img.gif">   <span>image name</span>   </a>',
        '<a href="http://example.de/epages/img.gif">   <span>image name</span>   </a>',
        'preserve whitespace' );

    $Test->( '<a href="img.gif">Teddybär</a>',
        '<a href="http://example.de/epages/img.gif">Teddybär</a>',
        'preserve umlauts' );

    $Test->( '<a href="#anchor">Internal Link</a>',
        '<a href="#anchor">Internal Link</a>',
        'preserve internal links' );

    $Test->( '<a href="img.gif">"Teddybär" ist &quot;Ted&quot; \'der\' &apos;Bär&apos;</a>',
        '<a href="http://example.de/epages/img.gif">&quot;Teddybär&quot; ist &quot;Ted&quot; &#39;der&#39; &#39;Bär&#39;</a>',
        'replace some XML entitles' );

    $Test->( "<script language=\'javascript\'>\nvar a = \'yes\';\n\tif( a < 2 )\n\t\talert(\"help\");\n</script>",
        "<script language=\"javascript\">\nvar a = \'yes\';\n\tif( a < 2 )\n\t\talert(\"help\");\n</script>",
        'don\'t replace XML entitles within script tags' );

    $aTags = ['img'];
    $Test->( '<a href="test.html"><img src="/WebRoot/test.png" /></a>',
        '<a href="test.html"><img src="http://example.de/WebRoot/test.png" /></a>',
        'replace uris with full url only for special tags.' );
};
